/*********************************************************************NVMH3****
File:  $Id: //devrel/NV_SDK_DOCS/Presentations/2005/Siggraph_2005/Shaders/SpyGirlHair.fx#2 $

Copyright NVIDIA Corporation 2005
TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED
*AS IS* AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS
OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS
BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES
WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.


Comments:
    Hair surface with Kajiya-Kay highlighting and some subsurface backlight.

******************************************************************************/

float Script : STANDARDSGLOBAL <
    string UIWidget = "none";
    string ScriptClass = "object";
    string ScriptOrder = "standard";
    string ScriptOutput = "color";
    string Script = "Technique=hair;";
> = 0.8;

#define KK_KD 0.2
#define KK_KS 0.3
#define KK_PHONG_EXPON 18.0
#define KK_GRAY_ONLY
#include <include\\kajiya_kay.fxh>

//
// Define "PIXEL_TAN" to use user-directable vectors for hair shaping,
//	rather than the predefined Tangent/Binormal. This allows shading
//	on surfaces with less-than-perfect parameterizations, or gently
//	animating the inputs to "HairUp" (see below) can provide apparent
//	motion in the hair
//
#define PIXEL_TAN

/************* TWEAKABLES **************/

float4x4 WorldIT : WorldInverseTranspose < string UIWidget="None"; >;
float4x4 WorldViewProj : WorldViewProjection < string UIWidget="None"; >;
float4x4 World : World < string UIWidget="None"; >;
float4x4 ViewInv : ViewInverse < string UIWidget="None"; >;

float4 LightPos : Position
<
    string Object = "PointLight";
    string Space = "World";
> = {100.0f, 100.0f, -100.0f, 0.0f};

float4 LightColor
<
	string UIWidget = "Color";
> = {1.0f, 1.0f, 1.0f, 1.0f};

float4 AmbiColor : Ambient
<
    string UIName =  "Ambient Light";
> = {0.07f, 0.07f, 0.07f, 1.0f};

float4 SurfColor : DIFFUSE
<
    string UIName =  "Surface";
    string UIWidget = "Color";
> = {1.0f, 1.0f, 1.0f, 1.0f};

float4 SheenColor 
<
    string UIName =  "Sheen";
    string UIWidget = "Color";
> = {0.4f, 0.2f, 0.05f, 1.0f};

float SpecExpon : SpecularPower
<
    string UIWidget = "slider";
    float UIMin = 1.0;
    float UIMax = 128.0;
    float UIStep = 1.0;
    string UIName =  "specular power";
> = 12.0;

float Ks
<
    string UIWidget = "slider";
    float UIMin = 0.0;
    float UIMax = 1.0;
    float UIStep = 0.01;
    string UIName =  "Specular Strength";
> = 1.0;

float RollOff
<
    string UIWidget = "slider";
    float UIMin = 0.0;
    float UIMax = 0.99;
    float UIStep = 0.01;
    string UIName =  "Subsurface Rolloff Range";
> = 0.2;

float HRollOff
<
    string UIWidget = "slider";
    float UIMin = 0.0;
    float UIMax = 0.99;
    float UIStep = 0.01;
    string UIName =  "Hilight Rolloff Range";
> = 0.2;

bool FlipY <
	string UIName = "Flip Texture Y (OpenGL)";
> = true;

#ifdef PIXEL_TAN
float HairUpX <
	string UIName = "Hair Flow Aim X";
	string UIWidget = "slider";
	float UIMin = -1.0;
	float UIMax = 1.0;
	float UIStep = 0.01;
> = 0.0f;

float HairUpY <
	string UIName = "Hair Flow Aim Y";
	string UIWidget = "slider";
	float UIMin = -1.0;
	float UIMax = 1.0;
	float UIStep = 0.01;
> = 1.0f;

float HairUpZ <
	string UIName = "Hair Flow Aim Z";
	string UIWidget = "slider";
	float UIMin = -1.0;
	float UIMax = 1.0;
	float UIStep = 0.01;
> = 0.25f;

static float3 HairUp = normalize(float3(HairUpX,HairUpY,HairUpZ));
#endif /* PIXEL_TAN */

//////////

texture colorTexture : DIFFUSE
<
    string ResourceName = "default_color.dds";
    string ResourceType = "2D";
>;

sampler2D colorSampler = sampler_state
{
	Texture = <colorTexture>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
};

texture normalTexture : NORMAL
<
    string ResourceName = "HairDirs.tga";
    string ResourceType = "2D";
>;

sampler2D normalSampler = sampler_state
{
	Texture = <normalTexture>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
};

/************* DATA STRUCTS **************/

/* data from application vertex buffer */
struct appdata {
    float3 Position	: POSITION;
    float4 UV		: TEXCOORD0;
    float4 Normal	: NORMAL;
    float4 Tangent	: TANGENT0;
    float4 Binormal	: BINORMAL0;
};

/* data passed from vertex shader to pixel shader */
struct vertexOutput {
    float4 HPosition	: POSITION;
    float4 TexCoord	: TEXCOORD0;
    float3 LightVec	: TEXCOORD1;
    float3 WorldNormal	: TEXCOORD2;
    float3 WorldEyeVec	: TEXCOORD3;
    float3 WorldTangent	: TEXCOORD4;
    float3 WorldBinorm	: TEXCOORD5;
    float3 Extra	: TEXCOORD6;
};

/*********** vertex shader ******/

vertexOutput mainVS(appdata IN)
{
    vertexOutput OUT;
    float4 Po = float4(IN.Position.xyz,1.0);
    //float4 nT = IN.Tangent;
    //nT = faceforward(nT,-IN.Normal,nT);
    OUT.WorldNormal = mul(IN.Normal, WorldIT).xyz;
    OUT.WorldTangent = mul(IN.Tangent, WorldIT).xyz;
    OUT.WorldBinorm = mul(IN.Binormal, WorldIT).xyz;
    float3 Pw = mul(Po, World).xyz;
    OUT.LightVec = LightPos - Pw;
    OUT.TexCoord = IN.UV;
	if (FlipY) {
		OUT.TexCoord.y = 1.0 - IN.UV.y;
	}
    OUT.WorldEyeVec = normalize(ViewInv[3].xyz - Pw);
    OUT.HPosition = mul(Po, WorldViewProj);
    OUT.Extra = Pw;
    return OUT;
}

/********* pixel shader ********/

float4 mainPS(vertexOutput IN) : COLOR
{

    float4 map = tex2D(colorSampler,IN.TexCoord.xy);
    float2 strands = tex2D(normalSampler,IN.TexCoord.xy).xy-float2(.5,.5);
    float3 Ln = normalize(IN.LightVec);
    float3 Nn = normalize(IN.WorldNormal);
#ifdef PIXEL_TAN
	float3 Tn = cross(HairUp,Nn);
	float3 Bn = normalize(cross(Nn,Tn));
#else /* !PIXEL_TAN */
    float3 Tn = normalize(IN.WorldTangent);
    float3 Bn = normalize(IN.WorldBinorm);
#endif /* !PIXEL_TAN */
    float3 Sn = (strands.x * Tn + strands.y * Bn);
    Sn = normalize(Sn);
    float3 Vn = normalize(IN.WorldEyeVec);
    float3 Hn = normalize(Vn + Ln);
    float ldn = dot(Ln,Nn);
    float subLamb = smoothstep(-RollOff,1.0,ldn);
    subLamb = max(0.0,subLamb);
    float subKK = smoothstep(-HRollOff,1.0,ldn); 
    subKK = max(0.0,subKK);
    float hdn = dot(Hn,Nn);
    float4 lighting = lit(ldn,hdn,SpecExpon);
    float4 diffContrib = SurfColor * map * (subLamb*LightColor + AmbiColor);
    float4 specContrib = Ks * lighting.z * LightColor;
    float4 result = diffContrib + specContrib;
    float TdL = 0.5+(dot(Sn,Ln)*0.5);
    float TdE = 0.5-(dot(Sn,Vn)*0.5);
    float kk =  tex2D(KKGraySamp,float2(TdL,TdE)).x;
    result = result + (subKK * kk * SheenColor);
    return float4(result.xyz,map.w);	// for alpha test
}

/*************/

technique hair <
	string Script = "Pass=p0;";
> {
	pass p0  <
		string Script = "Draw=geometry;";
	> {		
		// same vertex shader for both DX9 techniques
		VertexShader = compile vs_2_0 mainVS();
		ZEnable = true;
		ZWriteEnable = true;
		CullMode = None;
		AlphaTestEnable = True;
		AlphaRef = 0x0011;
		AlphaFunc = GreaterEqual;
		PixelShader = compile ps_2_a mainPS();
	}
}

/***************************** eof ***/

