/*********************************************************************NVMH3****

File:  $Id: //devrel/NV_SDK_DOCS/Presentations/2005/Siggraph_2005/Shaders/Translucent2.fx#2 $

Copyright NVIDIA Corporation 2005
TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED
*AS IS* AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS
OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS
BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES
WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

Comments:
    This shader models translucent materials, and shows light
	    shadowing/diffusing through the model.
    Shadow using a floating-point texture -- place depth into the
	r channel, but also a fresnel term (relative to the light position)
	in the b. When rendering, use the depth of the shaded point, relative
	to the lamp, to determine if it's in shadow and HOW MUCH it's in shadow
	based on diffusion of light through the surface. Diminish the intensity
	based on the light-space fresnel term to realistically model the loss
	of penetrating light along glancing angles.

******************************************************************************/

#include <include\\nvMatrix.fxh>
#include <include\\Quad.fxh>

float Script : STANDARDSGLOBAL <
	string UIWidget = "none";
	string ScriptClass = "scene";
	string ScriptOrder = "standard";
	string ScriptOutput = "color";
	string Script = "Technique=Main;";
> = 0.8; // version #

float4 ClearColor <string UIWidget="none";> = {0,0,0,0};
float4 ShadowClearColor <string UIWidget="none";> = {0,0,0,0};
float ClearDepth <string UIWidget="none";> = 1;

/************* "UN-TWEAKABLES," TRACKED BY CPU APPLICATION **************/

float4x4 WorldITXf : WorldInverseTranspose <string UIWidget="None";>;
float4x4 WorldViewProjXf : WorldViewProjection <string UIWidget="None";>;
float4x4 WorldXf : World <string UIWidget="None";>;
float4x4 ViewIXf : ViewInverse <string UIWidget="None";>;
float4x4 WorldViewITXf : WorldViewInverseTranspose <string UIWidget="None";>;
float4x4 WorldViewXf : WorldView <string UIWidget="None";>;
float4x4 ViewXf : View <string UIWidget="None";>;
float4x4 ViewITXf : ViewInverseTranspose <string UIWidget="None";>;

float4x4 LampViewXf : View <
    //string UIWidget="None";
    string frustum = "light0";
>;

float4x4 LampProjXf : Projection <
    //string UIWidget="None";
    string frustum = "light0";
>;

///////////////////////////////////////////////////////////////
/// TWEAKABLES ////////////////////////////////////////////////
///////////////////////////////////////////////////////////////

////////////////////////////////////////////// surface

float InternalPower
<
    string UIWidget = "slider";
    float UIMin = 0.1;
    float UIMax = 8.0;
    float UIStep = 0.01;
    string UIName = "Internal Exponent";
> = 0.5;

float InternalScale
<
    string UIWidget = "slider";
    float UIMin = 0.01;
    float UIMax = 5.0;
    float UIStep = 0.01;
    string UIName = "Internal Falloff Scale";
> = 2.0;

float Ki
<
    string UIWidget = "slider";
    float UIMin = 0.0;
    float UIMax = 20.5;
    float UIStep = 0.01;
    string UIName = "Internal Strength";
> = 1.0;

float3 InternalColor
<
	string UIWidget = "Color";
    string UIName = "Internal";
> = {0.7f, 0.7f, 1.0f};

float Inset
<
    string UIWidget = "slider";
    float UIMin = 0.01;
    float UIMax = 10.4;
    float UIStep = 0.01;
    string UIName = "Internal Inset Distance";
> = 0.03;

//////////////

float FresExp <
    string UIWidget = "slider";
    float UIMin = 0.1;
    float UIMax = 8.1;
    float UIStep = 0.01;
    string UIName = "Fresnel Exponent";
> = 0.5;

/////////////

float3 SurfColor : Diffuse
<
    string UIName = "Surface";
	string UIWidget = "Color";
> = {1.0f, 0.7f, 0.3f};

float Kd
<
    string UIWidget = "slider";
    float UIMin = 0.0;
    float UIMax = 1.5;
    float UIStep = 0.01;
    string UIName = "Diffuse";
> = 1.0;

float Ks
<
    string UIWidget = "slider";
    float UIMin = 0.0;
    float UIMax = 1.5;
    float UIStep = 0.01;
    string UIName = "Specular";
> = 1.0;


float SpecExpon : SpecularPower
<
    string UIWidget = "slider";
    float UIMin = 1.0;
    float UIMax = 128.0;
    float UIStep = 1.0;
    string UIName = "Specular power";
> = 52.0;

///////////////////

float ShadDensity <
	string UIName = "Shadow Density";
	string UIWidget = "slider";
	float UIMin = 0.0;
	float UIMax = 1.0;
	float UIStep = 0.1;
> = 1;

float ShadBias <
    string UIWidget = "slider";
    float UIMin = 0.0;
    float UIMax = 0.1;
    float UIStep = 0.001;
    string UIName = "Shad Bias";
> = 0.004;

////////////////////////////////////////////// spot light

float3 SpotLightPos : POSITION <
	string UIName = "Spot Pos";
	string Object = "SpotLight";
	string Space = "World";
> = {-1.0f, 1.0f, 0.0f};

float3 SpotLightDir : DIRECTION <
	string UIName = "Spot Dir";
	string Object = "SpotLight";
	string Space = "World";
> = {-0.707f, 0.707f, 1.0f};

float SpotLightCone <
    string UIWidget = "slider";
    float UIMin = 0.0;
    float UIMax = 90.5;
    float UIStep = 0.1;
    string UIName = "Spot Cone Angle";
> = 41.0;

float3 SpotLightColor : Diffuse <
	string UIName = "Spot Color";
	string Object = "SpotLight";
	string UIWidget = "Color";
> = {0.8f, 1.0f, 0.4f};

float SpotLightIntensity <
	string UIName = "Spot Intensity";
	string UIWidget = "slider";
	float UIMin = 0.0;
	float UIMax = 8;
	float UIStep = 0.1;
> = 1;


////////////////////////////////////////////// ambient light

float3 AmbiLightColor : Ambient
<
    string UIName = "Ambient Light";
> = {0.07f, 0.07f, 0.07f};

////////////////////////////////////////////////////////
/// TEXTURES ///////////////////////////////////////////
////////////////////////////////////////////////////////

#define SHADOW_SIZE 1024

// fixed-size shadow texture

texture ShadMap : RENDERCOLORTARGET < 
    // float2 ViewPortDimensions = {1.0,1.0};
	int Width = SHADOW_SIZE;
	int Height = SHADOW_SIZE;
    //int MIPLEVELS = 0;
    string format = "a16b16g16r16f";
    string UIWidget = "None";
>;

sampler ShadSampler = sampler_state {
    texture = <ShadMap>;
    AddressU  = BORDER;        
    AddressV  = BORDER;
    MIPFILTER = NONE;
    MINFILTER = POINT;
    MAGFILTER = POINT;
};

texture ShadDepthTarget : RENDERDEPTHSTENCILTARGET < 
	int Width = SHADOW_SIZE;
	int Height = SHADOW_SIZE;
    string format = "D24S8";
    string UIWidget = "None";
>;

/////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
/// SHADER CODE BEGINS /////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////

/*********************************************************/
/************* SHARED DATA STRUCT ************************/
/*********************************************************/

/* data from application vertex buffer */
struct appdata {
    float3 Position	: POSITION;
    float4 UV		: TEXCOORD0;
    float4 Normal	: NORMAL;
};

///////////////////////////////////////////////////////////////////////////////
/// Basic Version Does Almost All Calcs in the Pixel Shader ///////////////////
///////////////////////////////////////////////////////////////////////////////

struct vertexOutput {
    float4 HPosition	: POSITION;
    float2 UV		: TEXCOORD0;
    float3 LightVec	: TEXCOORD1;
    float3 WNormal	: TEXCOORD2;
    float3 WView		: TEXCOORD3;
    float4 LP		: TEXCOORD4;
    float4 LPin		: TEXCOORD5;
};

/*********************************************************/
/*********** vertex shader *******************************/
/*********************************************************/

////////////////////////////////////////////////////////////////////////////////
/// Vertex Shaders /////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

// from lamp POV
vertexOutput shadVS(appdata IN,
					uniform float4x4 SpotViewProjXf)
{
    vertexOutput OUT = (vertexOutput)0;
    OUT.WNormal = mul(IN.Normal,WorldITXf).xyz;
    float4 Po = float4(IN.Position.xyz,(float)1.0);	// object coordinates
    float4 Pw = mul(Po,WorldXf);			// world coordinates
    float4 Pl = mul(Pw,SpotViewProjXf);  // "P" in light coords

    OUT.LP = Pl;		// view coords

    OUT.WView = normalize(ViewIXf[3].xyz - Pw.xyz);	// obj coords
    OUT.HPosition = Pl; // mul(Po,WorldViewProjXf);	// screen clipspace coords
    OUT.UV = IN.UV.xy;
    OUT.LightVec = SpotLightPos - Pw.xyz; // in world coords
    return OUT;
}

// from scene camera POV
vertexOutput mainCamVS(appdata IN,
					uniform float4x4 SpotViewProjXf)
{
    vertexOutput OUT = (vertexOutput)0;
    float3 Nw = normalize(mul(IN.Normal,WorldITXf).xyz);
    OUT.WNormal = Nw;
    float4 Po = float4(IN.Position.xyz,(float)1.0);	// object coordinates
    float4 Pw = mul(Po,WorldXf);			// world coordinates
    float4 Pl = mul(Pw,SpotViewProjXf);  // "P" in light coords
    OUT.LP = Pl;		// view coords
	float4 Iw = Pw + -Inset*float4(Nw,0); // inset point
    float4 Il = mul(Iw,SpotViewProjXf);  // "P" in light coords
    OUT.LPin = Il;		// view coords

    OUT.WView = normalize(ViewIXf[3].xyz - Pw.xyz);	// obj coords
    OUT.HPosition = mul(Po,WorldViewProjXf);	// screen clipspace coords
    OUT.UV = IN.UV.xy;
    OUT.LightVec = SpotLightPos - Pw.xyz; // in world coords
    return OUT;
}

/*********************************************************/
/*********** pixel shader ********************************/
/*********************************************************/

float4 makeShadPS(vertexOutput IN,uniform float CosSpotAng) : COLOR
{
    float3 Nn = normalize(IN.WNormal);
    float3 Ln = normalize(IN.LightVec);
    float ldn = abs(dot(Ln,Nn));
    float fres = pow(ldn,FresExp);
    float dep = IN.LP.z;
    return float4(dep,dep,fres,1);
}

float4 useShadowPS(vertexOutput IN,uniform float CosSpotAng) : COLOR
{
    float3 Nn = normalize(IN.WNormal);
    float3 Vn = normalize(IN.WView);
    float3 Ln = normalize(IN.LightVec);
    float3 Hn = normalize(Vn + Ln);
    float hdn = dot(Hn,Nn);
    float ldn = dot(Ln,Nn);
    float aldn = abs(ldn);
    float ildn = -min(ldn,0);
    float ildn2 = pow(ildn,0.8);
    float4 litVec = lit(ldn,hdn,SpecExpon);
    ldn = litVec.y * SpotLightIntensity;
	float dl = dot(-SpotLightDir,Ln);
	dl = max((float)0,((dl-CosSpotAng)/(((float)1.0)-CosSpotAng)));
	ldn *= dl;
    float3 ambiContrib = SurfColor * AmbiLightColor;
    float3 diffContrib = SurfColor*(Kd*ldn * SpotLightColor);
    float3 specContrib = ((ldn * litVec.z * Ks) * SpotLightColor);
    float3 result = diffContrib + specContrib;
    // return float4(result,1);
    float2 nuv = float2(.5,-.5)*IN.LP.xy/IN.LP.w + float2(.5,.5);
    float4 tv = tex2D(ShadSampler,nuv);
    float shad = tv.x;
    float depth = (IN.LP.z) - ShadBias;
    //shad = ShadBias+ShadNear + shad*(ShadFar-ShadNear);
    float shadowed = (shad>depth)*ShadDensity+(1.0-ShadDensity);
	float3 litValue = lerp(float3(0,0,0),result,shadowed)+ambiContrib;
    float indepth = (IN.LPin.z/IN.LPin.w) - ShadBias;
	float internal = (1.0+max(0,indepth-shad))*InternalScale;
	internal =  SpotLightIntensity*dl*Ki/pow(internal,InternalPower);
    float3 backlit = internal*InternalColor;
    return float4(litValue+backlit,1);
 }

////////////////////////////////////////////////////////////////////
/// TECHNIQUES /////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////

technique Main <
	string Script = "Pass=MakeShadow;"
					"Pass=UseShadow;";
> {
	pass MakeShadow <
		string Script = "RenderColorTarget0=ShadMap;"
						"RenderDepthStencilTarget=ShadDepthTarget;"
						"RenderPort=light0;"
						"ClearSetColor=ShadowClearColor;"
						"ClearSetDepth=ClearDepth;"
						"Clear=Color;"
						"Clear=Depth;"
						"Draw=geometry;";
	> {
		VertexShader = compile vs_2_0 shadVS(mul(LampViewXf,LampProjXf));
		ZEnable = true;
		ZWriteEnable = true;
		CullMode = None;
		PixelShader = compile ps_2_a makeShadPS(cos(SpotLightCone*(float)(3.141592/180.0)));
	}
	pass UseShadow <
		string Script = "RenderColorTarget0=;"
						"RenderDepthStencilTarget=;"
						"RenderPort=;"
						"ClearSetColor=ClearColor;"
						"ClearSetDepth=ClearDepth;"
						"Clear=Color;"
						"Clear=Depth;"
						"Draw=geometry;";
	> {
		VertexShader = compile vs_2_0 mainCamVS(mul(LampViewXf,LampProjXf));
		ZEnable = true;
		ZWriteEnable = true;
		CullMode = None;
		PixelShader = compile ps_2_a useShadowPS(cos(SpotLightCone*(float)(3.141592/180.0)));
	}
}

//////// final view, four previous views as panels

/***************************** eof ***/
