/*********************************************************************NVMH3****
File:  $Id: //devrel/NV_SDK_DOCS/Presentations/2005/Siggraph_2005/Shaders/lambSkin_asperity.fx#2 $

Copyright NVIDIA Corporation 2005
TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED
*AS IS* AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS
OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS
BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES
WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.


Comments:
    A lambertian-like surface with light "bleed-through" -- appropriate
    for soft translucent materials like skin. The "subColor" represents
    the tinting acquired by light diffused below the surface.
	Set the "rollOff" angle to the cosine of the angle used for
    additional lighting "wraparound" -- the diffuse effect propogates based
    on the angle of LightDirection versus SurfaceNormal.
    	Versions are provided for shading in pixel or vertex shaders,
    textured or untextured.

******************************************************************************/

float Script : STANDARDSGLOBAL <
    string UIWidget = "none";
    string ScriptClass = "object";
    string ScriptOrder = "standard";
    string ScriptOutput = "color";
    string Script = "Technique=TexturedPS;";
> = 0.8;

#define KK_KS 0.3
#include "include\\kajiya_kay.fxh"
// #include "kajiya_kay.fxh"

/************* UN-TWEAKABLES **************/

float4x4 WorldIT : WorldInverseTranspose < string UIWidget="None"; >;
float4x4 WorldViewProj : WorldViewProjection < string UIWidget="None"; >;
float4x4 World : World < string UIWidget="None"; >;
float4x4 ViewIXf : ViewInverse < string UIWidget="None"; >;

/************* TWEAKABLES *****************/

float3 Up = {0,1,0};

float UpDown
<
    string UIWidget = "slider";
    float UIMin = -2.0;
    float UIMax = 2.00;
    float UIStep = 0.01;
    string UIName =  "Up and down w/hairs";
> = 0.0;
static float3 Deflector = (Up * UpDown);

float4 LightPos : POSITION
<
	string Object = "PointLight";
    string UIName =  "Lamp Position";
	string Space = "World";
> = {1403.0f, 1441.0f, 1690.0f, 0.0f};

float4 AmbiColor : AMBIENT <
	string UIWidget = "Color";
    string UIName =  "Ambient";
>  = {0.1f, 0.1f, 0.1f, 1.0f};

float4 DiffColor : DIFFUSE <
	string UIWidget = "Color";
    string UIName =  "Surface Diffuse";
>  = {0.9f, 1.0f, 0.9f, 1.0f};

float4 SubColor <
	string UIWidget = "Color";
    string UIName =  "Subsurface \"Bleed-thru\"";
>  = {1.0f, 0.2f, 0.2f, 1.0f};

float RollOff
<
    string UIWidget = "slider";
    float UIMin = 0.0;
    float UIMax = 0.99;
    float UIStep = 0.01;
    string UIName =  "Subsurface Rolloff Range";
> = 0.2;

float FuzzMinOpac
<
    string UIWidget = "slider";
    float UIMin = 0.0;
    float UIMax = 0.99;
    float UIStep = 0.01;
    string UIName =  "Min Fuzz Opacity";
> = 0.0;

float FuzzMaxOpac
<
    string UIWidget = "slider";
    float UIMin = 0.0;
    float UIMax = 1.0;
    float UIStep = 0.01;
    string UIName =  "Max Fuzz Opacity";
> = 0.2;

static float FuzzRange = (FuzzMaxOpac - FuzzMinOpac);

float FluffRollOff
<
    string UIWidget = "slider";
    float UIMin = 0.0;
    float UIMax = 0.99;
    float UIStep = 0.01;
    string UIName =  "Fluff Rolloff Range";
> = 0.5;

float4 FuzzColor <
	string UIWidget = "Color";
    string UIName =  "Fuzz";
>  = {1.0f, 1.0f, 0.8f, 1.0f};

/////////////// texture /////////////////

texture ColorTexture : DIFFUSE <
	string ResourceName = "DonHead2.jpg";
    string UIName =  "Skin Color";
	string ResourceType = "2D";
>;

sampler2D ColorSampler = sampler_state
{
	Texture = <ColorTexture>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
	AddressU = CLAMP;
	AddressV = CLAMP;
};

texture HairlineTex : DIFFUSE <
	string ResourceName = "DonHead2F.jpg";
    string UIName =  "Hairline";
	string ResourceType = "2D";
>;

sampler2D HairlineSampler = sampler_state
{
	Texture = <HairlineTex>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
	AddressU = CLAMP;
	AddressV = CLAMP;
};

texture FuzzTex : DIFFUSE <
	string ResourceName = "Pechfuzz_Obscure.tga";
    string UIName =  "Fuzz Visibility Tex";
	string ResourceType = "2D";
>;

sampler2D FuzzSampler = sampler_state
{
	Texture = <FuzzTex>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
	AddressU = CLAMP;
	AddressV = CLAMP;
};

/************* DATA STRUCTS **************/

/* data from application vertex buffer */
struct appdata {
    float3 Position	: POSITION;
    float4 UV		: TEXCOORD0;
    float4 Normal	: NORMAL;
};

/* data passed from vertex shader to pixel shader */
struct shadedVertexOutput {
    float4 HPosition	: POSITION;
    float4 TexCoord	: TEXCOORD0;
    float4 diffCol	: COLOR0;
};

/* data passed from vertex shader to pixel shader */
struct vertexOutput {
    float4 HPosition	: POSITION;
    float2 TexCoord		: TEXCOORD0;
    float3 LightVec		: TEXCOORD1;
    float3 WorldNormal	: TEXCOORD2;
    float3 ViewVec		: TEXCOORD3;
};

///////////////////////////////////////////////
// Shared "lambskin" diffuse function /////////
///////////////////////////////////////////////


//
// vectors are assumed to be normalized as needed
//
float fuzz_opacity(float3 N,
			  float3 Incident // may be view, or may be light direction
) {
    float idn = 1-abs(dot(Incident,N));
	float t = tex2D(FuzzSampler,float2(idn,0)).x;
	return FuzzMinOpac + (t*FuzzRange);
}

/////////////////////////

// these args must be pre-normalized
float fluff_vSine(float3 An, float3 Bn)
{
    float dotProd = dot(An,Bn);
    float theta = acos(dotProd);
    float result = sin(theta);
    return result;
}

// essentially Kajiya-Kay
//	again, vectors have been pre-normalized
float fluff_fiber_brdf(float Kd, float Ks,float phongExp,float3 Tn,float3 En,float3 Ln)
{
    float diff = Kd * fluff_vSine(Tn,Ln);
    float spec = Ks * pow((dot(Tn,Ln)*dot(Tn,En) +
	    fluff_vSine(Tn,Ln)*fluff_vSine(Tn,En)),phongExp);
    float result = (diff+spec);
    return result;
}

#if 0
// these are some older Cg functions I'm adapting

// mix backlighting with front -- vectors pre-normalized
float3 fluff_frontback_color(float3 reflColor,float3 tranColor,
	float3 Tn,float3 En,float3 Ln)
{
    float3 TxE = cross(Tn,En);
    float3 TxL = cross(Tn,Ln);
    float K = dot(TxE,TxL) / (length(TxE) * length(TxL));
    float3 result = (reflColor*(1.0f+K) + tranColor*(1.0f-K))/2.0f;
    return result;
}

//
// cheesy limiter
//
float fluff_rolloff(float weight,float cosRollMin,float cosRollMax,float3 norm, float3 Ln)
{
    float result = 1.0f + weight*(smoothstep(dot(norm,Ln),cosRollMin,cosRollMax)-1.0f);
    return result;
}

//
//// the fluff_opacity() function is used to determine self-shadowing in the fluff, and also
// to determine whether the underlying surface (blinn, plastic, etc) is obscured by the fluff
// 
// exp(12) == 162754 exp(7)=1096 exp(9)=8103
// but we will just use a texture for this to save GPU time
//
#define FLUFF_EPSILON 0.01f
#define FLUFF_EELIMIT 12.0f
float fluff_opacity_numeric(float maxFiberRadius,float minFiberRadius,float fiberLength,
    float D, float3 Tn, float3 Nn, float3 Vn)
{
    float Ah = fiberLength * ((maxFiberRadius+minFiberRadius)/2.0);
    float vdn = dot(Vn,Nn);
    float avdn = abs(vdn);
    float fluffOpacity = 0.0f;
    if (avdn>FLUFF_EPSILON) {
        float g = fluff_vSine(Vn,Tn)/vdn;
        float ee = D * Ah * g;
        fluffOpacity = exp(min(abs(ee),FLUFF_EELIMIT));
        fluffOpacity = 1.0f - 1.0f/min(10000.0f,fluffOpacity);
    }
    return fluffOpacity;
}

// general fluff illumination function
float3 fluff_illum_numeric (
    float3	cRefl,
    float3	cTran,
    float	Kd,		/* of the fluff */
    float	Ks,		/* of the fluff */
    float	phongExpon,	/* of the fluff */
    float	edgeWeight,	/* clipped, fluffRollOff */
    float	cosRollMin,
    float	cosRollMax,
    float	fluffDensity,
    float	selfShadowing,
    float	fiberLength,
    float	maxFiberRadius,
    float	minFiberRadius,
    float3	Tn,	/* Normalized tangent vect ("growth direction") */
    float3	Nf,	/* Normalized shading normal */
    float3	Vn,	/* Normalized view vector */
    float3	Ln,	/* Normalized light direction */
    float3	Cl	/* light color */
) {
    float lightOpacity = fluff_opacity_numeric(fiberLength,maxFiberRadius,minFiberRadius,fluffDensity,Tn,Nf,Ln);
    float3 fiberCl = Cl*(1.0f - selfShadowing*lightOpacity);
    float3 fiberColor = fluff_frontback_color(cRefl,cTran,Nf,Vn,Ln);
    float rollOff = fluff_rolloff(edgeWeight,cosRollMin,cosRollMax,Nf,Ln);
    // I called "Nf" not "Tn" -- hmmm, did I do this right?
    float brdf=fluff_fiber_brdf(Kd,Ks,phongExpon,Nf,Vn,Ln);
    float3 cLitFiber = fiberCl*fiberColor*brdf*rollOff;
    return cLitFiber;
}

// general fluff illumination function
float3 fluff_illum_simplified (
    float3	cRefl,
    float3	cTran,
    float	Kd,		/* of the fluff */
    float	Ks,		/* of the fluff */
    float	phongExpon,	/* of the fluff */
    float	edgeWeight,	/* clipped, fluffRollOff */
    float	cosRollMin,
    float	cosRollMax,
    float	fluffDensity,
    float	selfShadowing,
    float	fiberLength,
    float	maxFiberRadius,
    float	minFiberRadius,
    float3	Tn,	/* Normalized tangent vect ("growth direction") */
    float3	Nf,	/* Normalized shading normal */
    float3	Vn,	/* Normalized view vector */
    float3	Ln,	/* Normalized light direction */
    float3	Cl	/* light color */
) {
    float lightOpacity = fluff_opacity_numeric(fiberLength,maxFiberRadius,minFiberRadius,fluffDensity,Tn,Nf,Ln);
    float3 fiberCl = Cl*(1.0f - selfShadowing*lightOpacity);
    float3 fiberColor = fluff_frontback_color(cRefl,cTran,Nf,Vn,Ln);
    float rollOff = fluff_rolloff(edgeWeight,cosRollMin,cosRollMax,Nf,Ln);
    // I called "Nf" not "Tn" -- hmmm, did I do this right?
    float brdf=fluff_fiber_brdf(Kd,Ks,phongExpon,Nf,Vn,Ln);
    float3 cLitFiber = fiberCl*fiberColor*brdf*rollOff;
    return cLitFiber;
}

#endif /* zero */

//////////////////////////

/*********** vertex shader ******/

vertexOutput simpleVS(appdata IN) {
    vertexOutput OUT = (vertexOutput)0;
    float3 normal = normalize(IN.Normal);
    OUT.WorldNormal = mul(normal, WorldIT).xyz;
    float4 Po = float4(IN.Position.xyz,1);
    float3 Pw = mul(Po, World).xyz;
    OUT.LightVec = normalize(LightPos - Pw);
    OUT.TexCoord = IN.UV;
    OUT.HPosition = mul(Po, WorldViewProj);
    OUT.ViewVec = normalize(ViewIXf[3].xyz - Pw.xyz);	// world coords
    return OUT;
}

/********* pixel shader ********/

float fuzzy(float3 Ln, float3 Tn, float3 Vn)
{
    // "Tn" is the growth dir, often just "Nn"
    float TdL = 0.5+(dot(Tn,Ln)*0.5);
    float TdE = 0.5-(dot(Tn,Vn)*0.5);
    float kk = tex2D(KKGraySamp,float2(TdL,TdE)).x;
    return kk;
}
float4 fuzzyC(float3 Ln, float3 Tn, float3 Vn)
{
    // "Tn" is the growth dir, often just "Nn"
    float TdL = 0.5+(dot(Tn,Ln)*0.5);
    float TdE = 0.5-(dot(Tn,Vn)*0.5);
    float4 kk = tex2D(KKColorSamp,float2(TdL,TdE)).x;
    return kk;
}


float4 lambPS_t(vertexOutput IN) : COLOR {
    float3 Ln = normalize(IN.LightVec);
    float3 Nn = normalize(IN.WorldNormal);
    float3 Vn = normalize(IN.ViewVec);
    float3 Tn = Nn;
    Tn = normalize(Tn + Deflector);
    float ldn = dot(Ln,Nn);
    float diffComp = max(0,ldn);
    float4 diffContrib = float4((diffComp * DiffColor).xyz,1);
    float subLamb = smoothstep(-RollOff,1.0,ldn) - smoothstep(0.0,1.0,ldn);
    subLamb = max(0.0,subLamb);
    float4 subContrib = subLamb * SubColor;
    float subFluff = smoothstep(-FluffRollOff,1.0,ldn);
    subFluff = max(0.0,subFluff);
	//float hairy = fuzzy(Ln,TNn,Vn);
	//float4 hC = FuzzColor*hairy*subFluff;
	float4 hC = fuzzyC(Ln,Tn,Vn) *subFluff;
	float4 litC = diffContrib + AmbiColor + subContrib;
	float4 skinC = (litC*tex2D(ColorSampler,IN.TexCoord.xy));
	float hairline = tex2D(HairlineSampler,IN.TexCoord.xy).x;
	float fv = hairline * fuzz_opacity(Nn,Vn);
	return lerp(skinC,hC,fv);
}

/*************/

technique TexturedPS <
	string Script = "Pass=p0;";
> {
	pass p0 <
	string Script = "Draw=geometry;";
> {		
		VertexShader = compile vs_1_1 simpleVS();
		ZEnable = true;
		ZWriteEnable = true;
		CullMode = None;
		PixelShader = compile ps_2_0 lambPS_t();
	}
}

/***************************** eof ***/
