/*********************************************************************NVMH3****
File:  $Id: //devrel/NV_SDK_DOCS/Presentations/2005/Siggraph_2005/Shaders/post_shadowOverlayGradientBg.fx#2 $

Copyright NVIDIA Corporation 2005
TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED
*AS IS* AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS
OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS
BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES
WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

In this example, the surface is using its regular assigned shader (can be anythign)
 with a shadowing overlay.

******************************************************************************/

#define MAX_SHADOW_BIAS 0.0025
#define SHADOW_SIZE 1024

// Scene constants /////////////////////////////////////////////////

#include <include\\nvMatrix.fxh>
#include <include\\Quad.fxh>
#include <include\\shadowMap.fxh>

float Script : STANDARDSGLOBAL <
    string UIWidget = "none";
    string ScriptClass = "scene";
    string ScriptOrder = "postprocess";
    string ScriptOutput = "color";
    string Script = "Technique=AmbiShadow;";
> = 0.8;

float4 ClearColor <
	string UIWidget = "color";
	string UIName = "Background";
> = {0,0,0,0.0};

float4 ShadClearColor <
	string UIWidget = "none";
> = {1,1,1,0.0};

float ClearDepth <string UIWidget = "none";> = 1.0;

DECLARE_SHADOW_XFORMS("light0",LampViewXf,LampProjXf,ShadowViewProjXf)
DECLARE_SHADOW_BIAS
DECLARE_SHADOW_MAPS(ColorShadMap,ColorShadSampler,ShadDepthTarget,ShadDepthSampler)

///////////////////////////////////////////////////////////////
/// TWEAKABLES ////////////////////////////////////////////////
///////////////////////////////////////////////////////////////
float3 SpotLightPos : POSITION <
	string UIName = "Light Posistion";
	string Object = "SpotLight";
	string Space = "World";
> = {-1.0f, 1.0f, 0.0f};

float3 LightColor <
	string UIName = "Lighter Color";
	string Object = "PointLight";
	string UIWidget = "Color";
> = {1.0f, 1.0f, 1.0f};

float3 DarkColor <
	string UIName = "Darker Color";
	string Object = "PointLight";
	string UIWidget = "Color";
> = {0.8f, 1.0f, 0.4f};

float Intensity <
    string UIName = "Light Intensity";
    string UIWidget = "slider";
    float UIMin = 0;
    float UIMax = 1000.0;
> = 1.;

////////////////////////////////////////////////////////
/// TEXTURES ///////////////////////////////////////////
////////////////////////////////////////////////////////

DECLARE_QUAD_TEX(SceneMap,SceneSampler,"A8R8G8B8")
DECLARE_QUAD_DEPTH_BUFFER(SceneDepthTarget,"D24S8")

/////////////////////////////////////////////////////////////

/************* "UN-TWEAKABLES," TRACKED BY CPU APPLICATION **************/

float4x4 WorldITXf : WorldInverseTranspose <string UIWidget="None";>;
float4x4 WorldViewProjXf : WorldViewProjection <string UIWidget="None";>;
float4x4 WorldXf : World <string UIWidget="None";>;
float4x4 ViewIXf : ViewInverse <string UIWidget="None";>;
float4x4 WorldViewITXf : WorldViewInverseTranspose <string UIWidget="None";>;
float4x4 WorldViewXf : WorldView <string UIWidget="None";>;
float4x4 ViewXf : View <string UIWidget="None";>;
float4x4 ViewITXf : ViewInverseTranspose <string UIWidget="None";>;

////////////////////////////////////////////////////////////////////////////
/// SHADER CODE BEGINS /////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////

/*********************************************************/
/************* SHARED DATA STRUCT ************************/
/*********************************************************/

/* data from application vertex buffer */
struct appdata {
    float3 Position	: POSITION;
    float4 UV		: TEXCOORD0;
    float4 Normal	: NORMAL;
};

///////////////////////////////////////////////////////////////////////////////
/// Basic Version Does Almost All Calcs in the Pixel Shader ///////////////////
///////////////////////////////////////////////////////////////////////////////

struct OverlayVertexOutput {
    float4 HPosition	: POSITION;
    float4 UV		: TEXCOORD0;
    float4 LProj	: TEXCOORD1;
    float4 WPos	: TEXCOORD2;
};

////////////////////////////////////////////////////////////////////////////////
/// Vertex Shaders /////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

OverlayVertexOutput shadowUse2DVS(ShadowAppData IN,
		uniform float4x4 WorldXform,
		uniform float4x4 WorldITXform,
		uniform float4x4 WVPXform,
		uniform float4x4 ShadowVPXform,
		uniform float4x4 BiasXform) {
    OverlayVertexOutput OUT = (OverlayVertexOutput)0;
    // OUT.WNormal = mul(IN.Normal,WorldITXform).xyz; // world coords
    float4 Po = float4(IN.Position.xyz,(float)1.0);	// "P" in object coordinates
    float4 Pw = mul(Po,WorldXform);			// "P" in world coordinates
    float4 Pl = mul(Pw,ShadowVPXform);  // "P" in light coords
    OUT.LProj = mul(Pl,BiasXform);				// bias to make texcoord
    OUT.HPosition = mul(Po,WVPXform);	// screen clipspace coords
    OUT.UV = OUT.HPosition;
    OUT.WPos = Pw;
   return OUT;
}

/*********************************************************/
/*********** pixel shader ********************************/
/*********************************************************/

float4 useShadowPS(OverlayVertexOutput IN) : COLOR
{
    float4 sceneColor = tex2D(SceneSampler,IN.UV.xy);
    float4 shadowed = tex2Dproj(ShadDepthSampler,IN.LProj);
    float3 delta = IN.WPos.xyz - SpotLightPos;
    float ds = Intensity / dot(delta,delta);
    float4 nuv = float4((0.5*IN.UV.x/IN.UV.w)+0.5,
    					(0.5*IN.UV.y/IN.UV.w)+0.5,
    					0,1);
    float4 orig = tex2D(SceneSampler,float2(nuv.x,1.0-nuv.y));
    float3 shadTone = ds *lerp(DarkColor,LightColor,shadowed);
	return float4((shadTone*orig.xyz),1);
}

////////////////////////////////////////////////////////////////////
/// TECHNIQUES /////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////

technique AmbiShadow <
    	string Script =
						"Pass=MakeShadow;"
    					"RenderColorTarget0=SceneMap;"
    					"RenderDepthStencilTarget=SceneDepthTarget;"
	        			"ClearSetColor=ClearColor;"
	        			"ClearSetDepth=ClearDepth;"
   						"Clear=Color0;"
						"Clear=Depth0;"
						"RenderPort=;"
    					"ScriptExternal=color;"
						"Pass=UseShadow;";
> {
	pass MakeShadow <
		string Script = "RenderColorTarget0=ColorShadMap;"
						"RenderDepthStencilTarget=ShadDepthTarget;"
						"RenderPort=light0;"
						"ClearSetColor=ClearColor;"
						"ClearSetDepth=ClearDepth;"
						"Clear=Color;"
						"Clear=Depth;"
						"Draw=geometry;";
	> {
		VertexShader = compile vs_2_0 shadowGenVS(WorldXf,WorldITXf,ShadowViewProjXf);
		ZEnable = true;
		ZWriteEnable = true;
		ZFunc = LessEqual;
		CullMode = None;
		// no pixel shader
	}
	pass UseShadow <
		string Script = "RenderColorTarget0=;"
						"RenderDepthStencilTarget=;"
						"RenderPort=;"
						"ClearSetColor=ClearColor;"
						"ClearSetDepth=ClearDepth;"
						"Clear=Color;"
						"Clear=Depth;"
						"Draw=geometry;";
	> {
		//VertexShader = compile vs_2_0 mainCamVS();
		VertexShader = compile vs_2_0 shadowUse2DVS(WorldXf,WorldITXf, WorldViewProjXf,
								ShadowViewProjXf,ShadBiasXf);
		ZEnable = true;
		ZWriteEnable = true;
		ZFunc = LessEqual;
		CullMode = None;
		PixelShader = compile ps_2_a useShadowPS();
	}
}

/***************************** eof ***/
